/****************************************************************************
** Meta object code from reading C++ file 'executer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kturtle-26.07.80/src/interpreter/executer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'executer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8ExecuterE_t {};
} // unnamed namespace

template <> constexpr inline auto Executer::qt_create_metaobjectdata<qt_meta_tag_ZN8ExecuterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Executer",
        "currentlyExecuting",
        "",
        "TreeNode*",
        "node",
        "variableTableUpdated",
        "name",
        "Value",
        "value",
        "functionTableUpdated",
        "parameters",
        "getX",
        "double&",
        "getY",
        "ask",
        "QString&",
        "message",
        "text",
        "getDirection",
        "reset",
        "clear",
        "center",
        "go",
        "goX",
        "goY",
        "forward",
        "backward",
        "direction",
        "turnLeft",
        "turnRight",
        "penWidth",
        "penUp",
        "penDown",
        "penColor",
        "canvasColor",
        "canvasSize",
        "spriteShow",
        "spriteHide",
        "print",
        "fontSize",
        "stopWaiting"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentlyExecuting'
        QtMocHelpers::SignalData<void(TreeNode *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'variableTableUpdated'
        QtMocHelpers::SignalData<void(const QString &, const Value &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'functionTableUpdated'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QStringList, 10 },
        }}),
        // Signal 'getX'
        QtMocHelpers::SignalData<void(double &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'getY'
        QtMocHelpers::SignalData<void(double &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'ask'
        QtMocHelpers::SignalData<void(QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 8 },
        }}),
        // Signal 'message'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'getDirection'
        QtMocHelpers::SignalData<void(double &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'reset'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clear'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'center'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'go'
        QtMocHelpers::SignalData<void(double, double)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 }, { QMetaType::Double, 2 },
        }}),
        // Signal 'goX'
        QtMocHelpers::SignalData<void(double)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'goY'
        QtMocHelpers::SignalData<void(double)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'forward'
        QtMocHelpers::SignalData<void(double)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'backward'
        QtMocHelpers::SignalData<void(double)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'direction'
        QtMocHelpers::SignalData<void(double)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'turnLeft'
        QtMocHelpers::SignalData<void(double)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'turnRight'
        QtMocHelpers::SignalData<void(double)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'penWidth'
        QtMocHelpers::SignalData<void(double)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'penUp'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'penDown'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'penColor'
        QtMocHelpers::SignalData<void(double, double, double)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 }, { QMetaType::Double, 2 }, { QMetaType::Double, 2 },
        }}),
        // Signal 'canvasColor'
        QtMocHelpers::SignalData<void(double, double, double)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 }, { QMetaType::Double, 2 }, { QMetaType::Double, 2 },
        }}),
        // Signal 'canvasSize'
        QtMocHelpers::SignalData<void(double, double)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 }, { QMetaType::Double, 2 },
        }}),
        // Signal 'spriteShow'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spriteHide'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'print'
        QtMocHelpers::SignalData<void(const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'fontSize'
        QtMocHelpers::SignalData<void(double)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'stopWaiting'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Executer, qt_meta_tag_ZN8ExecuterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Executer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ExecuterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ExecuterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8ExecuterE_t>.metaTypes,
    nullptr
} };

void Executer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Executer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentlyExecuting((*reinterpret_cast<std::add_pointer_t<TreeNode*>>(_a[1]))); break;
        case 1: _t->variableTableUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Value>>(_a[2]))); break;
        case 2: _t->functionTableUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 3: _t->getX((*reinterpret_cast<std::add_pointer_t<double&>>(_a[1]))); break;
        case 4: _t->getY((*reinterpret_cast<std::add_pointer_t<double&>>(_a[1]))); break;
        case 5: _t->ask((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1]))); break;
        case 6: _t->message((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->getDirection((*reinterpret_cast<std::add_pointer_t<double&>>(_a[1]))); break;
        case 8: _t->reset(); break;
        case 9: _t->clear(); break;
        case 10: _t->center(); break;
        case 11: _t->go((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 12: _t->goX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->goY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->forward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->backward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->direction((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->turnLeft((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->turnRight((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->penWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->penUp(); break;
        case 21: _t->penDown(); break;
        case 22: _t->penColor((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 23: _t->canvasColor((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 24: _t->canvasSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 25: _t->spriteShow(); break;
        case 26: _t->spriteHide(); break;
        case 27: _t->print((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->fontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 29: _t->stopWaiting(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(TreeNode * )>(_a, &Executer::currentlyExecuting, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(const QString & , const Value & )>(_a, &Executer::variableTableUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(const QString & , const QStringList & )>(_a, &Executer::functionTableUpdated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double & )>(_a, &Executer::getX, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double & )>(_a, &Executer::getY, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(QString & )>(_a, &Executer::ask, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(const QString & )>(_a, &Executer::message, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double & )>(_a, &Executer::getDirection, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)()>(_a, &Executer::reset, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)()>(_a, &Executer::clear, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)()>(_a, &Executer::center, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double , double )>(_a, &Executer::go, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::goX, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::goY, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::forward, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::backward, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::direction, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::turnLeft, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::turnRight, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::penWidth, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)()>(_a, &Executer::penUp, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)()>(_a, &Executer::penDown, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double , double , double )>(_a, &Executer::penColor, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double , double , double )>(_a, &Executer::canvasColor, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double , double )>(_a, &Executer::canvasSize, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)()>(_a, &Executer::spriteShow, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)()>(_a, &Executer::spriteHide, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(const QString & )>(_a, &Executer::print, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (Executer::*)(double )>(_a, &Executer::fontSize, 28))
            return;
    }
}

const QMetaObject *Executer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Executer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ExecuterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Executer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 30;
    }
    return _id;
}

// SIGNAL 0
void Executer::currentlyExecuting(TreeNode * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Executer::variableTableUpdated(const QString & _t1, const Value & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void Executer::functionTableUpdated(const QString & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Executer::getX(double & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Executer::getY(double & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Executer::ask(QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Executer::message(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Executer::getDirection(double & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Executer::reset()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Executer::clear()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Executer::center()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Executer::go(double _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void Executer::goX(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Executer::goY(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Executer::forward(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Executer::backward(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Executer::direction(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void Executer::turnLeft(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void Executer::turnRight(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Executer::penWidth(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Executer::penUp()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void Executer::penDown()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void Executer::penColor(double _t1, double _t2, double _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2, _t3);
}

// SIGNAL 23
void Executer::canvasColor(double _t1, double _t2, double _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1, _t2, _t3);
}

// SIGNAL 24
void Executer::canvasSize(double _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1, _t2);
}

// SIGNAL 25
void Executer::spriteShow()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void Executer::spriteHide()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void Executer::print(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}

// SIGNAL 28
void Executer::fontSize(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}
QT_WARNING_POP
