/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.12.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // kturtleui.rc
  0x0,0x0,0x2,0xb6,
  0x28,
  0xb5,0x2f,0xfd,0x60,0xfe,0x9,0x65,0x15,0x0,0x96,0x9e,0x57,0x1d,0x20,0x57,0x1e,
  0xa7,0x25,0x89,0xdf,0x65,0x9d,0xd0,0x1e,0x17,0x7b,0x61,0xd4,0xe9,0xbb,0x15,0x7f,
  0x7,0x99,0x36,0x1,0xf2,0x1,0x40,0x3,0x4e,0x3,0x58,0x0,0x4b,0x0,0x4b,0x0,
  0x8e,0x2,0xb5,0x32,0x4e,0x14,0x53,0xe4,0x4,0x8c,0x37,0xed,0x53,0xc,0xb6,0x32,
  0x7e,0xd6,0x2,0x30,0xdc,0x3d,0x2a,0xaf,0x85,0x2,0x20,0xd8,0xca,0xf7,0xfa,0x21,
  0xb6,0x58,0x91,0x63,0x6,0x1f,0x12,0xd8,0xb2,0x48,0x36,0x9a,0x8e,0x66,0xc5,0x5a,
  0xbd,0xbb,0x19,0x5a,0x12,0x4c,0x67,0x3,0xb9,0x44,0x2,0x14,0x80,0xd1,0x60,0x89,
  0xc1,0xe7,0x2b,0x5b,0x54,0x1c,0x61,0x3f,0xde,0xc5,0xc,0x84,0x3,0x5b,0x70,0x47,
  0xea,0x4b,0x5a,0x52,0x64,0x6a,0x50,0x1,0xd2,0xb5,0x7f,0x2b,0x97,0xee,0x53,0x37,
  0x4f,0xb3,0xd9,0xd7,0xb7,0xde,0x90,0x6b,0xdb,0xca,0xd7,0xd2,0xc,0x63,0x1d,0x3d,
  0x47,0x9f,0x52,0xf4,0xa0,0xd6,0xdf,0xde,0x7b,0x9a,0xd6,0x35,0xb6,0x24,0x47,0xb7,
  0xfd,0x56,0x5,0x1a,0xa5,0x6b,0x4f,0x63,0x45,0x42,0x57,0x4e,0x91,0x33,0xed,0xf3,
  0xbe,0x75,0xc6,0x3b,0xbb,0xb0,0xbc,0xdf,0xb9,0x9b,0xab,0x29,0x2c,0x56,0x5a,0x6e,
  0xdc,0xf1,0x5f,0x1c,0xff,0x97,0xfa,0x86,0x5e,0x9c,0xcd,0x5b,0xb6,0x2f,0xed,0x6,
  0xbd,0xb6,0x1e,0xa1,0xec,0xde,0xc8,0xee,0x9f,0xfb,0x4f,0x7d,0x4b,0x2d,0xa9,0x94,
  0xb5,0xc1,0x7d,0xf7,0x22,0x5c,0xa8,0x59,0x70,0x77,0x52,0x6f,0x8f,0x5b,0x5a,0xdd,
  0xe4,0x9a,0xe1,0xea,0x15,0x89,0x56,0x3c,0x21,0x8e,0xc4,0xee,0xed,0x47,0x2f,0xf9,
  0x92,0xfa,0x39,0x39,0x49,0xeb,0xf3,0x4d,0xcb,0x8d,0x41,0x8e,0x1f,0x19,0x0,0x46,
  0xac,0x55,0xa1,0x40,0xb5,0xfd,0xfa,0xd6,0xb4,0xb2,0x29,0xf7,0xe3,0xea,0xee,0x3c,
  0x8b,0xa3,0xb3,0x94,0x96,0x17,0xdb,0xea,0xcf,0x16,0xa0,0x87,0x91,0xb3,0x18,0x47,
  0x27,0xa2,0x0,0x8d,0x7b,0xca,0xa5,0x4c,0xb9,0x20,0xea,0xeb,0x11,0x55,0x6e,0xcb,
  0x51,0x7a,0xbb,0xef,0xdb,0x49,0x39,0x32,0xef,0x8f,0x70,0x5b,0xa9,0x41,0xb4,0xb2,
  0x7e,0x46,0x7f,0x5b,0xb9,0x5a,0xb1,0x93,0x9a,0x2a,0x80,0x88,0xa0,0x81,0x11,0x73,
  0xce,0xcd,0x8c,0x24,0x49,0x61,0xd9,0xd,0x20,0x44,0x40,0x90,0x64,0x45,0x6e,0xad,
  0x48,0x40,0x6d,0xfd,0x53,0x4e,0x26,0x50,0x2c,0xaf,0x21,0x8c,0x2c,0x88,0x59,0x16,
  0x7f,0x31,0x48,0x7e,0x9c,0x6,0xa9,0x69,0x83,0x34,0x82,0xbe,0x1c,0x3d,0x52,0x13,
  0x21,0x61,0xf1,0x70,0x6d,0x10,0xcb,0xb5,0x92,0x92,0x26,0x41,0x36,0xfb,0x90,0xb5,
  0x2e,0x32,0xb9,0x67,0x6d,0x50,0x18,0x36,0x94,0x46,0xca,0xb3,0xe5,0xa6,0x2c,0x1d,
  0x69,0x9f,0x20,0x4b,0x6c,0x38,0x30,0x38,0x9a,0xb4,0xbf,0x7d,0xa9,0x43,0x10,0xd1,
  0x98,0x5,0x43,0x21,0xd2,0xc8,0x54,0x8d,0x1c,0x27,0x2f,0x55,0x56,0xfe,0x92,0x45,
  0x64,0x38,0xc7,0xc5,0xc,0xe1,0xb2,0x8e,0x28,0x39,0x2a,0xc1,0x7b,0x8a,0x29,0xd0,
  0xb0,0x70,0x65,0x18,0xda,0x4c,0xe5,0x47,0x5f,0xcd,0x44,0xc6,0x2f,0x57,0x2,0xf7,
  0x88,0x92,0x1,0x39,0x26,0xcd,0xe2,0x4d,0x78,0x26,0x40,0xa4,0x85,0xc2,0x8c,0xa8,
  0xc0,0x2c,0x1b,0xc1,0x20,0x45,0x8c,0x28,0xfd,0xea,0x66,0x15,0xd8,0xc,0x41,0xd5,
  0x46,0xd4,0xe1,0xa8,0xec,0x7a,0x64,0x1,0xcb,0x20,0x3a,0xea,0x30,0x7c,0x82,0xe3,
  0x16,0xb1,0xb5,0xfb,0x3e,0x16,0x5a,0x2f,0x4,0xae,0x5c,0xfe,0xff,0xee,0x5a,0x85,
  0x9c,0x84,0x38,0x48,0x8d,0x7f,0x53,0xbe,0x69,0xe2,0xf5,0xe6,0x87,0x4c,0x8,0x99,
  0xd5,0xe1,0xb6,0x21,0x87,0xfb,0x88,0x8c,0xdb,0x12,0x1f,0xd7,0x8c,0x6a,0x60,0x40,
  0x8f,0x2f,0x17,0x59,0x40,0x10,0x7,0x1f,0xc6,0x89,0x81,0x49,0x55,0x66,0x83,0x9a,
  0x90,0x18,0x86,0xc0,0x88,0x32,0xde,0xf4,0x9b,0x19,0xbd,0xdf,0xdc,0x55,0x4b,0xc9,
  0xd2,0x59,0x51,0x8f,0xd6,0x9,0x6,0xdc,0xe1,0x60,0xae,0x6a,0x2b,0x36,0x9b,0xa7,
  0x1d,0x8f,0x96,0xa2,0xe4,0xda,0x28,0x7c,0x68,0x6b,0x8a,0x72,0x94,0x19,0xc2,0x80,
  0x4b,0x60,0xe7,0xa0,0x7c,0x39,0xdc,0x36,0x91,0x1,0x44,0xd4,0xfa,0xad,0x6f,0x7,
  0x8f,0x40,0x4,0xaf,0x2,
  
};

static const unsigned char qt_resource_name[] = {
  // kxmlgui5
  0x0,0x8,
  0xf,0x42,0xe5,0x85,
  0x0,0x6b,
  0x0,0x78,0x0,0x6d,0x0,0x6c,0x0,0x67,0x0,0x75,0x0,0x69,0x0,0x35,
    // kturtle
  0x0,0x7,
  0x2,0xbc,0x9b,0xc5,
  0x0,0x6b,
  0x0,0x74,0x0,0x75,0x0,0x72,0x0,0x74,0x0,0x6c,0x0,0x65,
    // kturtleui.rc
  0x0,0xc,
  0xc,0x9c,0x56,0xe3,
  0x0,0x6b,
  0x0,0x74,0x0,0x75,0x0,0x72,0x0,0x74,0x0,0x6c,0x0,0x65,0x0,0x75,0x0,0x69,0x0,0x2e,0x0,0x72,0x0,0x63,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9f,0x70,0xaa,0xce,0x70,
  // :/kxmlgui5
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9f,0x70,0xaa,0xce,0x70,
  // :/kxmlgui5/kturtle
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9f,0x70,0xaa,0xce,0x70,
  // :/kxmlgui5/kturtle/kturtleui.rc
  0x0,0x0,0x0,0x2a,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9f,0x70,0xaa,0xce,0x70,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_kturtle)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_kturtle)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kturtle)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kturtle)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_kturtle)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_kturtle)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
