/*
    SPDX-FileCopyrightText: 2003-2006 Cies Breijs <cies AT kde DOT nl>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

// the purpose of this file is to make the auto-moc system of cmake work

#include "echoer.h"

#include "moc_echoer.cpp"
