/*
    SPDX-FileCopyrightText: 2003-2006 Cies Breijs <cies AT kde DOT nl>

    SPDX-License-Identifier: GPL-2.0-only
*/


//BEGIN GENERATED gui_connect_inc CODE

/* The code between the line that start with "//BEGIN GENERATED" and "//END GENERATED"
 * is generated by "generate.rb" according to the definitions specified in
 * "definitions.rb". Please make all changes in the "definitions.rb" file, since all
 * all change you make here will be overwritten the next time "generate.rb" is run.
 * Thanks for looking at the code!
 */

	connect(executer, &Executer::reset, 
		canvas, &Canvas::slotReset);
	connect(executer, &Executer::clear, 
		canvas, &Canvas::slotClear);
	connect(executer, &Executer::center, 
		canvas, &Canvas::slotCenter);
	connect(executer, &Executer::go, 
		canvas, &Canvas::slotGo);
	connect(executer, &Executer::goX, 
		canvas, &Canvas::slotGoX);
	connect(executer, &Executer::goY, 
		canvas, &Canvas::slotGoY);
	connect(executer, &Executer::forward, 
		canvas, &Canvas::slotForward);
	connect(executer, &Executer::backward, 
		canvas, &Canvas::slotBackward);
	connect(executer, &Executer::direction, 
		canvas, &Canvas::slotDirection);
	connect(executer, &Executer::turnLeft, 
		canvas, &Canvas::slotTurnLeft);
	connect(executer, &Executer::turnRight, 
		canvas, &Canvas::slotTurnRight);
	connect(executer, &Executer::penWidth, 
		canvas, &Canvas::slotPenWidth);
	connect(executer, &Executer::penUp, 
		canvas, &Canvas::slotPenUp);
	connect(executer, &Executer::penDown, 
		canvas, &Canvas::slotPenDown);
	connect(executer, &Executer::penColor, 
		canvas, &Canvas::slotPenColor);
	connect(executer, &Executer::canvasColor, 
		canvas, &Canvas::slotCanvasColor);
	connect(executer, &Executer::canvasSize, 
		canvas, &Canvas::slotCanvasSize);
	connect(executer, &Executer::spriteShow, 
		canvas, &Canvas::slotSpriteShow);
	connect(executer, &Executer::spriteHide, 
		canvas, &Canvas::slotSpriteHide);
	connect(executer, &Executer::print, 
		canvas, &Canvas::slotPrint);
	connect(executer, &Executer::fontSize, 
		canvas, &Canvas::slotFontSize);

//END GENERATED gui_connect_inc CODE

	connect(executer, &Executer::getX,
		canvas, &Canvas::getX);
	connect(executer, &Executer::getY,
		canvas, &Canvas::getY);
	connect(executer, &Executer::getDirection,
		canvas, &Canvas::getDirection);
	connect(executer, &Executer::message,
		this, &MainWindow::slotMessageDialog);
	connect(executer, &Executer::ask,
		this, &MainWindow::slotInputDialog);
