/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kturtle.Interpreter.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "interpreteradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class InterpreterAdaptor
 */

InterpreterAdaptor::InterpreterAdaptor(Interpreter *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

InterpreterAdaptor::~InterpreterAdaptor()
{
    // destructor
}

bool InterpreterAdaptor::encounteredErrors()
{
    // handle method call org.kde.kturtle.Interpreter.encounteredErrors
    return parent()->encounteredErrors();
}

QStringList InterpreterAdaptor::getErrorStrings()
{
    // handle method call org.kde.kturtle.Interpreter.getErrorStrings
    return parent()->getErrorStrings();
}

void InterpreterAdaptor::initialize(const QString &inputString)
{
    // handle method call org.kde.kturtle.Interpreter.initialize
    parent()->initialize(inputString);
}

void InterpreterAdaptor::interpret()
{
    // handle method call org.kde.kturtle.Interpreter.interpret
    parent()->interpret();
}

int InterpreterAdaptor::state()
{
    // handle method call org.kde.kturtle.Interpreter.state
    return parent()->state();
}


#include "moc_interpreteradaptor.cpp"
